#========================================================================================
# package      DE_EPAGES::EasyMarketing::t::unit::OAuth_RegistrationLinkBuilder
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::EasyMarketing::API::RegistrationLinkBuilder
#========================================================================================
package DE_EPAGES::EasyMarketing::t::unit::OAuth_RegistrationLinkBuilder;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::EasyMarketing::API::OAuth::RegistrationLinkBuilder;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall );
use Test::Exception;
use Test::More;

use strict;



sub test {
    my $self = shift;
    $self->test_BuildUriWithStandardRegistrationLink();
    $self->test_BuildUriWithRegistrationLinkThatContainsParams();
    $self->test_BuildUriWithStandardRegistrationLink_ButNoProviderID();
    done_testing();
}



sub test_BuildUriWithStandardRegistrationLink {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hSystemAttributes ={
        'URL' => 'http://register.test',
        'ProviderID' => 'TestProviderID',
    };
    my $RegistrationLinkBuilder = $self->_getRegistrationLinkBuilder($hSystemAttributes);
    my $ObjectWithShop =  $self->_getAnEpagesObjectWithAShop();


    my $RegistrationLink = $RegistrationLinkBuilder->buildRegistrationLink($ObjectWithShop);


    is($RegistrationLink, 'http://register.test?backlink=http%3A%2F%2FmyShop.url&providerID=TestProviderID&shopApi=http%3A%2F%2Frest-api-host%3A8080%2Frs%2Fshops%2FOAuthTestShop', "$SubTestName - test if correct registration link was built"); ## no critic (ProhibitLongLines)

    throws_ok { GetParametersFromMockifyCall($RegistrationLinkBuilder, '_LogDebug'); } (
        qr/_LogDebug was not called: _LogDebug was/,
        "$SubTestName - test if log communication was not called"
    );

    my ($Object, $hParams, $hOptions) = @ {GetParametersFromMockifyCall($RegistrationLinkBuilder, '_BuildShopUrl')};
    is(
        $Object->{'LocalISA'},
        'DE_EPAGES::Product::API::Object::Product',
        "$SubTestName - test if log _BuildShopUrl gets the correct object"
    );
    is_deeply(
        $hParams,
        {'ViewAction' => 'MBO-ViewEasyMarketingAuthorization'},
        "$SubTestName - test if log _BuildShopUrl gets the correct hParams"
    );
    is_deeply(
        $hOptions,
        {'Type' => 'admin', 'UseSSL' => 1},
        "$SubTestName - test if log _BuildShopUrl gets the correct hOptions"
    );
}



sub test_BuildUriWithRegistrationLinkThatContainsParams {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hSystemAttributes ={
        'URL' => 'http://register.test?param=test',
        'ProviderID' => 'TestProviderID',
    };
    my $RegistrationLinkBuilder = $self->_getRegistrationLinkBuilder($hSystemAttributes);
    my $ObjectWithShop =  $self->_getAnEpagesObjectWithAShop();

    my $RegistrationLink = $RegistrationLinkBuilder->buildRegistrationLink($ObjectWithShop);


    is($RegistrationLink, 'http://register.test?param=test&backlink=http%3A%2F%2FmyShop.url&providerID=TestProviderID&shopApi=http%3A%2F%2Frest-api-host%3A8080%2Frs%2Fshops%2FOAuthTestShop', "$SubTestName - test if backlink param was appended"); ## no critic (ProhibitLongLines)

    throws_ok { GetParametersFromMockifyCall($RegistrationLinkBuilder, '_LogDebug'); } (
        qr/_LogDebug was not called: _LogDebug was/,
        "$SubTestName - test if log communication was not called"
    );

    my ($Object, $hParams, $hOptions) = @ {GetParametersFromMockifyCall($RegistrationLinkBuilder, '_BuildShopUrl')};
    is(
        $Object->{'LocalISA'},
        'DE_EPAGES::Product::API::Object::Product',
        "$SubTestName - test if log _BuildShopUrl gets the correct object"
    );
    is_deeply(
        $hParams,
        {'ViewAction' => 'MBO-ViewEasyMarketingAuthorization'},
        "$SubTestName - test if log _BuildShopUrl gets the correct hParams"
    );
    is_deeply(
        $hOptions,
        {'Type' => 'admin', 'UseSSL' => 1},
        "$SubTestName - test if log _BuildShopUrl gets the correct hOptions"
    );
}



sub test_BuildUriWithStandardRegistrationLink_ButNoProviderID {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hSystemAttributes ={
        'URL' => 'http://register.test',
        'ProviderID' => undef,
    };
    my $RegistrationLinkBuilder = $self->_getRegistrationLinkBuilder($hSystemAttributes);
    my $ObjectWithShop =  $self->_getAnEpagesObjectWithAShop();


    my $RegistrationLink = $RegistrationLinkBuilder->buildRegistrationLink($ObjectWithShop);


    is(
        $RegistrationLink,
        'http://register.test?backlink=http%3A%2F%2FmyShop.url&providerID=&shopApi=http%3A%2F%2Frest-api-host%3A8080%2Frs%2Fshops%2FOAuthTestShop',
        "$SubTestName - test if correct registration link was built"
    );

    my $MockedCallParameters = GetParametersFromMockifyCall($RegistrationLinkBuilder, '_LogDebug');
    is_deeply(
        $MockedCallParameters,
        $self->_ExpectedLogEasyMarketingCommunicationParameters(),
        "$SubTestName - test if log communication gets the correct parameters"
    );

    my ($Object, $hParams, $hOptions) = @ {GetParametersFromMockifyCall($RegistrationLinkBuilder, '_BuildShopUrl')};
    is(
        $Object->{'LocalISA'},
        'DE_EPAGES::Product::API::Object::Product',
        "$SubTestName - test if log _BuildShopUrl gets the correct object"
    );
    is_deeply(
        $hParams,
        {'ViewAction' => 'MBO-ViewEasyMarketingAuthorization'},
        "$SubTestName - test if log _BuildShopUrl gets the correct hParams"
    );
    is_deeply(
        $hOptions,
        {'Type' => 'admin', 'UseSSL' => 1},
        "$SubTestName - test if log _BuildShopUrl gets the correct hOptions"
    );
}



#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _getRegistrationLinkBuilder {
    my $self = shift;
    my ($hSystemAttributes) = @_;

    ## no critic (ProhibitLongLines)
    my $MockedRegistrationLinkBuilder = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::EasyMarketing::API::OAuth::RegistrationLinkBuilder');
    $MockedRegistrationLinkBuilder->addMockWithReturnValueAndParameterCheck('_BuildShopUrl', 'http://myShop.url', ['object', 'hashref', 'hashref']);
    $MockedRegistrationLinkBuilder->addMockWithReturnValueAndParameterCheck('_LogDebug', undef, ['string', 'hashref', 'string']);
    $MockedRegistrationLinkBuilder->addMockWithReturnValue('_LoadRootObject', $self->_getMockedRootObject($hSystemAttributes));
    ## use critic (ProhibitLongLines)

    return $MockedRegistrationLinkBuilder->getMockObject();
}



sub _ExpectedLogEasyMarketingCommunicationParameters {
    my $self = shift;
    return [
          'buildRegistrationLink',
          {
                'Message' => 'Without provider ID it is not possible for EasyMarketing to enter the Rest service of epages', ## no critic (ProhibitLongLines)
                'ShopAlias' => 'OAuthTestShop',
                'Method' => '_getProviderID'
            },
          'DE_EPAGES::EasyMarketing::API::OAuth::RegistrationLinkBuilder',
        ];
}



sub _getMockedRootObject {
    my $self = shift;
    my ($hSystemAttributes) = @_;
    my $hAttributes = {
        'EasyMarketing_RegistrationLink' => $hSystemAttributes->{'URL'},
        'EasyMarketing_ProviderID' => $hSystemAttributes->{'ProviderID'},
        'EPagesJBaseURL' => 'http://rest-api-host:8080/rs/',
    };
    my $MockedSystem = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site', $hAttributes);
    return $MockedSystem;
}



sub _getAnEpagesObjectWithAShop {
    my $self = shift;
    my $hAttributes = {
        'Site' => $self->_getShop(),
    };
    my $MockedSystem = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::Product', $hAttributes); ## no critic (ProhibitLongLines)
    return $MockedSystem;
}



sub _getShop {
    my $self = shift;
    my ($hSystemAttributes) = @_;
    my $hAttributes = {
        'Alias' => 'OAuthTestShop',
    };
    my $MockedShop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    return $MockedShop;
}



__PACKAGE__->runScript();

1;