#========================================================================================
# package      DE_EPAGES::Product::t::ProductUpdate
# base         DE_EPAGES::Shop::API::TestImportXML
# state        public
#----------------------------------------------------------------------------------------
# description  Integration test for DE_EPAGES::Product::API::Resource::Product (Patch).
#========================================================================================
package DE_EPAGES::Product::t::ProductUpdate;
use base DE_EPAGES::Shop::API::TestImportXML;

use DE_EPAGES::Product::API::Resource::Product;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Exception;
use Test::More;

use strict;

# locale/language id of english
my $LocaleID = 'en_GB';
my $LanguageID = 2;
my $CurrencyID = 'EUR';
my $TaxModel = 1;

## no critic (ProhibitMagicNumbers)



sub shopAlias {
    return 'RestApiTestShop';
}



sub importXMLPath {
    return 'ShopXMLs/Shop_for_ProductResource_update_test.xml';
}



sub test {
    my $self = shift;

    $self->test_patchProduct_add();

    done_testing();
}



sub test_patchProduct_add {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $ProductFolder = $Shop->get('ProductFolder');

    my $Product = $ProductFolder->child('TestProduct');
    my $Resource = DE_EPAGES::Product::API::Resource::Product->new();

    my $hPatchRequest = [
        {
            'op' => 'add',
            'path' => '/productNumber',
            'value' => 'product-12345',
        },
        {
            'op' => 'add',
            'path' => '/name',
            'value' => 'testName123',
        },
        {
            'op' => 'add',
            'path' => '/shortDescription',
            'value' => 'this is a short description',
        },
        {
            'op' => 'add',
            'path' => '/description',
            'value' => '<b>Description</b>This is a great product. Like it <a href="vote.for.it/test">here</a>',
        },
        {
            'op' => 'add',
            'path' => '/manufacturer',
            'value' => 'This product was made by me',
        },
        {
            'op' => 'add',
            'path' => '/upc',
            'value' => '123456789999',
        },
        {
            'op' => 'add',
            'path' => '/essentialFeatures',
            'value' => 'buyable product',
        },
        {
            'op' => 'add',
            'path' => '/searchKeywords',
            'value' => ['search', 'keywords', 'product'],
        },
        {
            'op' => 'add',
            'path' => '/priceInfo/price/amount',
            'value' => 3.14,
        },
        {
            'op' => 'add',
            'path' => '/priceInfo/manufacturerPrice/amount',
            'value' => 2.71,
        },
        {
            'op' => 'add',
            'path' => '/priceInfo/ecoParticipationPrice/amount',
            'value' => 1.41,
        },
        {
            'op' => 'add',
            'path' => '/priceInfo/depositPrice/amount',
            'value' => 1.61,
        },
    ];
    my $hUpdatedProductData = $Resource->patch($Product, $LocaleID, undef, $hPatchRequest);

    is($hUpdatedProductData->{'productNumber'}, 'product-12345', "$SubTestName - test if updated productNumber of product was returned.");
    is($hUpdatedProductData->{'name'}, 'testName123', "$SubTestName - test if updated name of product was returned.");
    is($hUpdatedProductData->{'description'}, '<b>Description</b>This is a great product. Like it <a href="vote.for.it/test">here</a>', "$SubTestName - test if updated description of product was returned.");
    is($hUpdatedProductData->{'shortDescription'}, 'this is a short description', "$SubTestName - test if updated shortDescription of product was returned.");
    is($hUpdatedProductData->{'manufacturer'}, 'This product was made by me', "$SubTestName - test if updated manufacturer of product was returned.");
    is($hUpdatedProductData->{'upcEan'}, '123456789999', "$SubTestName - test if updated upc of product was returned.");
    is($hUpdatedProductData->{'essentialFeatures'}, 'buyable product', "$SubTestName - test if updated essentialFeatures of product were returned.");
    is_deeply($hUpdatedProductData->{'searchKeywords'}, ['search', 'keywords', 'product'], "$SubTestName - test if updated searchKeywords of product were returned.");
    is($hUpdatedProductData->{'priceInfo'}->{'price'}->{'amount'}, 3.14, "$SubTestName - test if updated price of product were returned.");
    is($hUpdatedProductData->{'priceInfo'}->{'manufacturerPrice'}->{'amount'}, 2.71, "$SubTestName - test if updated manufacturerPrice of product were returned.");
    is($hUpdatedProductData->{'priceInfo'}->{'ecoParticipationPrice'}->{'amount'}, 1.41, "$SubTestName - test if updated ecoParticipationPrice of product were returned.");
    is($hUpdatedProductData->{'priceInfo'}->{'depositPrice'}->{'amount'}, 1.61, "$SubTestName - test if updated depositPrice of product were returned.");

    is($Product->alias(), 'product-12345', "$SubTestName - test if productNumber of product was updated.");
    is($Product->get('Name', $LanguageID), 'testName123', "$SubTestName - test if name of product was updated.");
    is($Product->get('Text', $LanguageID), '<b>Description</b>This is a great product. Like it <a href="vote.for.it/test">here</a>', "$SubTestName - test if description of product was updated.");
    is($Product->get('Description', $LanguageID), 'this is a short description', "$SubTestName - test if shortDescription of product was updated.");
    is($Product->get('Manufacturer', $LanguageID), 'This product was made by me', "$SubTestName - test if manufacturer of product was updated.");
    is($Product->get('UPCEAN', $LanguageID), '123456789999', "$SubTestName - test if UPC of product was updated.");
    is($Product->get('MainCharacteristics', $LanguageID), 'buyable product', "$SubTestName - test if essentialFeatures of product were updated.");
    is($Product->get('Keywords', $LanguageID), 'search, keywords, product', "$SubTestName - test if searchKeywords of product was updated.");
    is($Product->getListPrice($CurrencyID, $TaxModel), 3.14, "$SubTestName - test if price of product was updated.");
    is($Product->getManufacturerPrice($CurrencyID, $TaxModel), 2.71, "$SubTestName - test if manufacturerPrice of product was updated.");
    is($Product->getEcoParticipation($CurrencyID, $TaxModel), 1.41, "$SubTestName - test if ecoParticipationPrice of product was updated.");
    is($Product->getDeposit($CurrencyID, $TaxModel), 1.61, "$SubTestName - test if depositPrice of product was updated.");
}

__PACKAGE__->runScript;