#========================================================================================
# package      DE_EPAGES::Product::t::unit::ProductPatch
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Product::API::ProductPatch
#========================================================================================
package DE_EPAGES::Product::t::unit::ProductPatch;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Product::API::ProductPatch;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::More;

use strict;

my $LanguageID = 2; # language id of english



sub test {
    my $self = shift;

    $self->test_add_productNumber();
    $self->test_add_name();
    $self->test_add_shortDescription();
    $self->test_add_description();
    $self->test_add_manufacturer();
    $self->test_add_upc();
    $self->test_add_ean();
    $self->test_add_essentialFeatures();
    $self->test_add_searchKeywords();
    $self->test_add_price();
    $self->test_add_manufacturerPrice();
    $self->test_add_ecoParticipationPrice();
    $self->test_add_depositPrice();

    $self->test_remove_name();
    $self->test_remove_shortDescription();
    $self->test_remove_description();
    $self->test_remove_manufacturer();
    $self->test_remove_upc();
    $self->test_remove_ean();
    $self->test_remove_essentialFeatures();
    $self->test_remove_searchKeywords();
    $self->test_remove_price();
    $self->test_remove_manufacturerPrice();
    $self->test_remove_ecoParticipationPrice();
    $self->test_remove_depositPrice();

    done_testing();
}



sub test_add_productNumber {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/productNumber',
        'value' => 'product-12345',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->alias(), 'product-12345', "$SubTestName - test if the attribute productNumber/Alias was set.");
}



sub test_add_name {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/name',
        'value' => 'new name',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('Name'), 'new name', "$SubTestName - test if the attribute name was set.");
}



sub test_add_shortDescription {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/shortDescription',
        'value' => 'new short description',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('Description'), 'new short description', "$SubTestName - test if the attribute shortDescription was set.");
}



sub test_add_description {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/description',
        'value' => 'new description',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('Text'), 'new description', "$SubTestName - test if the attribute description was set.");
}



sub test_add_energyLabelsString {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/energyLabelsString',
        'value' => 'new energy labels',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('EnergyLabelsString'), 'new energy labels', "$SubTestName - test if the attribute energyLabelsString was set.");
}



sub test_add_manufacturer {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/manufacturer',
        'value' => 'new manufacturer',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('Manufacturer'), 'new manufacturer', "$SubTestName - test if the attribute manufacturer was set.");
}



sub test_add_upc {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/upc',
        'value' => '123456789999',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('UPCEAN'), '123456789999', "$SubTestName - test if the attribute upc was set.");
}



sub test_add_ean {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/ean',
        'value' => '4006381333931',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('UPCEAN'), '4006381333931', "$SubTestName - test if the attribute ean was set.");
}



sub test_add_essentialFeatures {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/essentialFeatures',
        'value' => 'this is a nice product',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('MainCharacteristics'), 'this is a nice product', "$SubTestName - test if attribute essentialFeatures was set.");
}



sub test_add_searchKeywords {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/searchKeywords',
        'value' => ['search', 'keywords', 'product'],
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('Keywords'), 'search, keywords, product', "$SubTestName - test if attribute searchKeywords was set.");
}



sub test_add_price {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $CurrencyID = 'EUR';
    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID, $CurrencyID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/priceInfo/price/amount',
        'value' => '3.14',
    };

    $ProductPatch->patch($hPatchOperation);

    my $TaxModel = 1;
    my $ListPrice = $Product->getListPrice($CurrencyID, $TaxModel);
    is($ListPrice, '3.14', "$SubTestName - test if the attribute price was set.");
}



sub test_add_manufacturerPrice {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $CurrencyID = 'EUR';
    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID, $CurrencyID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/priceInfo/manufacturerPrice/amount',
        'value' => '2.71',
    };

    $ProductPatch->patch($hPatchOperation);

    my $TaxModel = 1;
    my $ListPrice = $Product->getManufacturerPrice($CurrencyID, $TaxModel);
    is($ListPrice, '2.71', "$SubTestName - test if the attribute manufacturerPrice was set.");
}



sub test_add_ecoParticipationPrice {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $CurrencyID = 'EUR';
    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID, $CurrencyID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/priceInfo/ecoParticipationPrice/amount',
        'value' => '1.41',
    };

    $ProductPatch->patch($hPatchOperation);

    my $TaxModel = 1;
    my $ListPrice = $Product->getEcoParticipation($CurrencyID, $TaxModel);
    is($ListPrice, '1.41', "$SubTestName - test if the attribute ecoParticipationPrice was set.");
}



sub test_add_depositPrice {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $CurrencyID = 'EUR';
    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID, $CurrencyID);

    my $hPatchOperation = {
        'op' => 'add',
        'path' => '/priceInfo/depositPrice/amount',
        'value' => '1.61',
    };

    $ProductPatch->patch($hPatchOperation);

    my $TaxModel = 1;
    my $ListPrice = $Product->getDeposit($CurrencyID, $TaxModel);
    is($ListPrice, '1.61', "$SubTestName - test if the attribute depositPrice was set.");
}



sub test_remove_name {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/name',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('Name'), undef, "$SubTestName - test if the attribute name was removed.");
}



sub test_remove_shortDescription {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/shortDescription',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('Description'), undef, "$SubTestName - test if the attribute short description was removed.");
}



sub test_remove_description {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/description',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('Text'), undef, "$SubTestName - test if the attribute description was removed.");
}



sub test_remove_energyLabelsString {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/energyLabelsString',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('EnergyLabelsString'), undef, "$SubTestName - test if the attribute energyLabelsString was removed.");
}



sub test_remove_manufacturer {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/manufacturer',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('Manufacturer'), undef, "$SubTestName - test if the attribute manufacturer was removed.");
}



sub test_remove_upc {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/upc',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('UPCEAN'), undef, "$SubTestName - test if the attribute upc was removed.");
}



sub test_remove_ean {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/ean',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('UPCEAN'), undef, "$SubTestName - test if the attribute ean was removed.");
}



sub test_remove_essentialFeatures {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/essentialFeatures',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('MainCharacteristics'), undef, "$SubTestName - test if attribute essentialFeatures was removed.");
}



sub test_remove_searchKeywords {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/searchKeywords',
    };

    $ProductPatch->patch($hPatchOperation);

    is($Product->get('Keywords'), undef, "$SubTestName - test if attribute searchKeywords was removed.");
}



sub test_remove_price {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $CurrencyID = 'EUR';
    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID, $CurrencyID);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/priceInfo/price/amount',
    };

    $ProductPatch->patch($hPatchOperation);

    my $TaxModel = 1;
    my $ListPrice = $Product->getListPrice($CurrencyID, $TaxModel);
    is($ListPrice, undef, "$SubTestName - test if attribute price was removed.");
}



sub test_remove_manufacturerPrice {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $CurrencyID = 'EUR';
    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID, $CurrencyID);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/priceInfo/manufacturerPrice/amount',
    };

    $ProductPatch->patch($hPatchOperation);

    my $TaxModel = 1;
    my $ListPrice = $Product->getManufacturerPrice($CurrencyID, $TaxModel);
    is($ListPrice, undef, "$SubTestName - test if attribute manufacturerPrice was removed.");
}



sub test_remove_ecoParticipationPrice {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $CurrencyID = 'EUR';
    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID, $CurrencyID);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/priceInfo/ecoParticipationPrice/amount',
    };

    $ProductPatch->patch($hPatchOperation);

    my $TaxModel = 1;
    my $ListPrice = $Product->getEcoParticipation($CurrencyID, $TaxModel);
    is($ListPrice, undef, "$SubTestName - test if attribute ecoParticipationPrice was removed.");
}



sub test_remove_depositPrice {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $CurrencyID = 'EUR';
    my $Product = $self->_createProduct();
    my $ProductPatch = $self->_createProductPatch($Product, $LanguageID, $CurrencyID);

    my $hPatchOperation = {
        'op' => 'remove',
        'path' => '/priceInfo/depositPrice/amount',
    };

    $ProductPatch->patch($hPatchOperation);

    my $TaxModel = 1;
    my $ListPrice = $Product->getDeposit($CurrencyID, $TaxModel);
    is($ListPrice, undef, "$SubTestName - test if attribute depositPrice was removed.");
}



sub _createProduct {
    my $self = shift;
    my $hAttributes = {
        'Site' => $self->_createShop(),
    };
    my $Product = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::Product', $hAttributes);

    $Product->mock('setListPrice', sub {
        my $MockedSelf = shift;
        my ($CurrencyId, $TaxModel, $Value) = @_;
        $MockedSelf->{'ListPrice'.$CurrencyId} = $Value;
        return;
    });
    $Product->mock('getListPrice', sub {
        my $MockedSelf = shift;
        my ($CurrencyId, $TaxModel) = @_;
        return $MockedSelf->{'ListPrice'.$CurrencyId};
    });

    $Product->mock('setManufacturerPrice', sub {
        my $MockedSelf = shift;
        my ($CurrencyId, $TaxModel, $Value) = @_;
        $MockedSelf->{'ManufacturerPrice'.$CurrencyId} = $Value;
        return;
    });
    $Product->mock('getManufacturerPrice', sub {
        my $MockedSelf = shift;
        my ($CurrencyId, $TaxModel) = @_;
        return $MockedSelf->{'ManufacturerPrice'.$CurrencyId};
    });

    $Product->mock('setEcoParticipation', sub {
        my $MockedSelf = shift;
        my ($CurrencyId, $TaxModel, $Value) = @_;
        $MockedSelf->{'EcoParticipation'.$CurrencyId} = $Value;
        return;
    });
    $Product->mock('getEcoParticipation', sub {
        my $MockedSelf = shift;
        my ($CurrencyId, $TaxModel) = @_;
        return $MockedSelf->{'EcoParticipation'.$CurrencyId};
    });

    $Product->mock('setDeposit', sub {
        my $MockedSelf = shift;
        my ($CurrencyId, $TaxModel, $Value) = @_;
        $MockedSelf->{'Deposit'.$CurrencyId} = $Value;
        return;
    });
    $Product->mock('getDeposit', sub {
        my $MockedSelf = shift;
        my ($CurrencyId, $TaxModel) = @_;
        return $MockedSelf->{'Deposit'.$CurrencyId};
    });

    return $Product;
}



sub _createShop {
    my $self = shift;
    my $hAttributes = {
        'TaxModel' => 1,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
}



sub _createProductPatch {
    my $self = shift;
    my ($Product, $LanguageId, $CurrencyId) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Product::API::ProductPatch',
        [$Product, $LanguageId, $CurrencyId]
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();