/*
 * @copyright        © Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module            de_epages/remotesearch/inc/resultpagehandler
 *
 * @revision        $Revision: 1.24 $
 */

/*jslint browser: true*/
/*global define*/

define("de_epages/remotesearch/inc/resultpagehandler", [
    "jquery",
    "ep",
    "jquery/i18n",
    "jquery/fn/class",
    "ep/ajax",
    "ep/fn/busy",
    "ep/ui/form"
], function ($, ep) {

    'use strict';

    return function (options) {

        var
            // Check browser feature
            pushStateSupport = window.history && history.pushState && history.replaceState && history.state !== undefined,

            // Init options
            o = $.extend({
                ajaxViewAction: "View",
                ajaxTemplateName: "Content",
                ajaxLanguageId: ep.config.languageId,
                ajaxPreventOnCategory: false,

                searchResultsId: "RemoteSearchResults",
                searchFacetsId: "RemoteSearchFacets",

                currency: $.i18n.settings.currency
            }, options),

            // Page content area
            page = $("body"),

            // Navbars (left / right)
            navbar = $(".NavBarLeft > .PropertyContainer > .SizeContainer, .NavBarRight > .PropertyContainer > .SizeContainer"),

            // Category tree box
            searchBox = navbar.children(".SearchBox, .SearchElement").first(),

            // Category tree box
            basketBox = navbar.children(".BasketBox").first(),

            afterBox,

            // Search results area
            results = $("#" + o.searchResultsId),

            // Search facets area
            facets = $("#" + o.searchFacetsId),

            // Form of facets
            facetsForm = facets.find("form"),

            firstPopstate,
            locationUrl,
            ajaxUrl,
            facetAjaxInit,
            facetAjaxReset,
            win,

            // Timeout id holder
            delayTimout,

            // Reset trigger delay for auto submit
            clearSubmit = function () {
                clearTimeout(delayTimout);
            },

            // Delayed auto sumbit
            delaySubmit = function () {
                clearSubmit();

                // Delay submit after user input handling
                delayTimout = setTimeout(function () {
                    facetsForm.trigger("submit");
                }, 1800);
            },

            onChange = function () {
                var elem = $(this);

                if (o.ajaxPreventOnCategory && elem.is(".RemoteSearchFacetCategory *")) {
                    // Reset AJAX setup
                    if (facetAjaxReset) {
                        facetAjaxReset();
                    }

                    // Change ObjectID for new category context
                    facetsForm.find("input[name=ObjectID]")
                        .val(elem.val());
                }

                facetsForm.trigger("submit");
            },

            onKeyup = function () {
                var input = $(this),
                    slide = input.closest("li")
                        .find(".ui-slider"),
                    val = input.val(),
                    value = $.i18n.parseNumber(val),
                    values = slide.slider("option", "values");
                values[input.is(".FacetRangeMin") ? 0 : 1] = value;

                // Set value to named (hidden) input
                input.prev()
                    .val(val);

                // Set value to slider
                if (!isNaN(value)) {
                    slide.slider("option", "values", values);
                }
            },

            onSlide = function () {
                var slide = $(this),
                    values = slide.slider("values"),
                    format = slide.data("format") || 'n',
                    vals = [$.i18n.formatNumber(values[0], format), $.i18n.formatNumber(values[1], format)],
                    input = slide.closest("li")
                        .find("input");

                input.filter(".FacetRangeMin")
                    .val(vals[0]);
                input.filter(".FacetRangeMax")
                    .val(vals[1]);

                input.trigger('focus') // Trigger non native placeholder handling
                    .trigger('validate'); // Validate values

                clearSubmit();
            },

            onSlidestop = function () {
                onSlide.apply(this, arguments);
                delaySubmit();
            },

            onReset = function () {
                var block = $(this)
                        .closest('.RemoteSearchFacet, form');

                block.find("input:checkbox, input:radio")
                    .prop("checked", false);

                block.find("input:text")
                    .val("")
                    .trigger("keyup");

                facetsForm.trigger("submit");
            };

        // Set factets area position
        if (navbar.length) {
            if (searchBox.length) {
                if ($.contains(navbar.first()[0], searchBox[0])) {
                    navbar = navbar.first();
                } else if ($.contains(navbar.last()[0], searchBox[0])) {
                    navbar = navbar.last();
                }
                afterBox = searchBox;
            } else if (basketBox.length) {
                navbar = navbar.first();

                if ($.contains(navbar.first()[0], basketBox[0])) {
                    afterBox = basketBox;
                }
            } else {
                navbar = navbar.first();
            }

            // Place facets
            if (afterBox) {
                afterBox.after(facets);

                // Remove category tree in the navbar where the facets are placed
                navbar.children(".CategoriesBox")
                    .first()
                    .remove();
            } else {
                navbar.prepend(facets);
            }

            facets.removeClass('visually-hidden');

            facets.addClass("NavBarElement" + (navbar.find("[class*='NavBarElement']").length + 1));
        } else {
            facets.prependTo('.ContentAreaWrapper');
        }

        // trigger event if search content was inserted
        $(document).trigger('facetssearch:ready');

        // Init AJAX refresh
        if (pushStateSupport) {
            firstPopstate = true;

            locationUrl = "";

            ajaxUrl = "";

            // Prepare facets form for AJAX load
            facetAjaxInit = function () {
                var self = facetAjaxInit;

                if (!self.cache) {
                    self.cache = $('<div/>');
                    self.jsonSnippet = $('<div><input type="hidden" name="TemplateAction" value="' + o.ajaxViewAction + '"/><input type="hidden" name="TemplateName" value="' + o.ajaxTemplateName + '"/><input type="hidden" name="TemplateLanguageID" value="' + o.ajaxLanguageId + '"/></div>');
                }
                self.viewAction = facetsForm.find("input[name=ViewAction]");

                // Clear
                self.viewAction.val(o.ajaxViewAction);
                self.jsonSnippet.prependTo(self.cache);

                // Set pushstate url
                locationUrl = "?" + facetsForm.serialize();

                // Set ajax properties
                self.viewAction.val("JSONSnippet");
                self.jsonSnippet.prependTo(facetsForm);

                ajaxUrl = "?" + facetsForm.serialize();
            };

            facetAjaxReset = function () {
                var self = facetAjaxInit;

                if (self.cache) {
                    // Clear
                    self.viewAction.val(o.ajaxViewAction);
                    self.jsonSnippet.prependTo(self.cache);
                }

                facetsForm.uiForm("option", "ajax", false);
            };

            win = $(window);

            // Init current ajax and location url
            facetAjaxInit();

            // Set current history data
            history.replaceState({
                snippetUrl: ajaxUrl
            }, null, location.href);

            // Register history event
            win.on("popstate", function () {
                if (firstPopstate) {
                    // Reset prent ajax on first popstate
                    firstPopstate = false;

                    return;
                }

                var state = history.state;

                // Hide busy layer on page
                page.busy("show");

                // Load state by ajax
                ep.ajax({
                    url: state.snippetUrl,
                    dataType: "json"
                })
                    .done(function (data) {
                        var snippet = $("<div/>")
                                .append(data.snippet);

                        // Replace page content
                        results.empty()
                            .html(snippet.find("#" + o.searchResultsId).html() || snippet.html() || "")
                            .find(".ep-js")
                            .metaparse();

                        facetsForm.empty()
                            .html(snippet.find("#" + o.searchFacetsId + " form").html() || "")
                            .find(".ep-js")
                            .metaparse();

                        // Derenference
                        snippet = undefined;

                        // Hide busy layer on page
                        page.busy("hide");

                        // trigger event if new content was inserted
                        $(document).trigger('facetssearch:loaded');
                    })
                    .fail(function () {
                        // Hide busy layer on page
                        page.busy("hide");
                    });
            });

            // Init form for auto AJAX
            facetsForm.uiForm({
                ajax: function () {
                    // Show busy layer on page
                    page.busy("show");

                    // Manipulate form for template loading via AJAX
                    facetAjaxInit();

                    // Push history state
                    history.pushState({
                        snippetUrl: ajaxUrl
                    }, null, locationUrl);

                    // Reset prent ajax on first popstate
                    firstPopstate = false;

                    // Trigger popstate for last push
                    win.trigger("popstate");
                }
            });
        } else {
            facetAjaxInit = function () {
                return;
            };

            // set hidden input for ViewAction to default Faceted-ViewAction if only "View" was found
            facetAjaxInit.viewAction = facetsForm.find("input[name=ViewAction]");

            if (facetAjaxInit.viewAction.val() === "View") {
                facetAjaxInit.viewAction.val("ViewFaceted");
            }

            facetsForm.uiForm();
        }

        // Register input event handling
        facetsForm.on("change", "input:not(:text)", onChange)
            .on("keyup", "input[name] + input:text", onKeyup)
            .on("slidestop", ".ui-slider", onSlidestop)
            .on("slide", ".ui-slider", onSlide)
            .on("click", "a.RemoteSearchFacetsReset", onReset);
    };

});